import asyncio
import logging

import aiohttp
import async_timeout

_LOGGER = logging.getLogger(__name__)


def _load_enum(enum, value, default=None):
    """Parse an enum with fallback."""
    try:
        return enum(value)
    except ValueError:
        return default


async def async_request(session, url, **kwargs):
    """Do a web request and manage response."""
    try:
        with async_timeout.timeout(10):
            _LOGGER.debug("Sending %s to %s", kwargs, url)
            response = await session(url, **kwargs)
        if response.status != 200:
            _LOGGER.error("HTTP status %d, response %s.",
                          response.status, (await response.text()))
            return False
        result = await response.json()
    except asyncio.TimeoutError:
        _LOGGER.error("Timeout getting SPC data from %s.", url)
        return False
    except aiohttp.ClientError:
        _LOGGER.error("Error getting SPC data from %s.", url)
        return False
    else:
        _LOGGER.debug("HTTP request response: %s", result)

    if result['status'] != 'success':
        _LOGGER.error(
            "SPC Web Gateway call unsuccessful for resource: %s", url)
        return False

    return result



import logging

from pyspcwebgw.const import ZoneInput, ZoneType, ZoneStatus
from pyspcwebgw.utils import _load_enum

_LOGGER = logging.getLogger(__name__)


class Zone:
    """Represents an SPC alarm system zone."""
    SUPPORTED_SIA_CODES = ('ZO', 'ZC', 'ZX', 'ZD', 'ZM', 'BA',
                           'BB', 'BU', 'BR', 'BC')

    def __init__(self, area, spc_zone):
        self._id = spc_zone['id']
        self._name = spc_zone['zone_name']
        self._area = area

        self.update(spc_zone)

    def __str__(self):
        return '{id}: {name} ({type}). Input: {inp}, status: {status}'.format(
            id=self.id, name=self.name, type=self.type,
            inp=self.input, status=self.status)

    @property
    def id(self):
        return self._id

    @property
    def name(self):
        return self._name

    @property
    def input(self):
        return self._input

    @property
    def type(self):
        return self._type

    @property
    def status(self):
        return self._status

    @property
    def area(self):
        return self._area

    def update(self, spc_zone):
        _LOGGER.debug("Update zone %s", self.id)

        self._input = _load_enum(ZoneInput, spc_zone['input'])
        self._type = _load_enum(ZoneType, spc_zone['type'])
        self._status = _load_enum(ZoneStatus, spc_zone['status'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

VERSION = '0.3.0'
DOWNLOAD_URL = \
    'https://github.com/mbrrg/pyspcwebgw/archive/{}.zip'.format(VERSION)

setup(
    name='pyspcwebgw',
    packages=['pyspcwebgw'],
    version=VERSION,
    description='A Python library for communicating with SPC Web Gateway.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Martin Berg',
    author_email='mbrrg@users.noreply.github.com',
    license='MIT',
    url='https://github.com/mbrrg/pyspcwebgw',
    download_url=DOWNLOAD_URL,
    install_requires=['aiohttp', 'asynccmd'],
    python_requires='>=3.5',
    scripts=['scripts/spcwebgw-console'],
    keywords=['spc', 'vanderbilt', 'web gateway'],
    classifiers=[
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Topic :: Home Automation'
    ],
)



import logging

from pyspcwebgw.const import AreaMode
from pyspcwebgw.utils import _load_enum

_LOGGER = logging.getLogger(__name__)


class Area:
    """Represents and SPC alarm system area."""
    SUPPORTED_SIA_CODES = ('CG', 'NL', 'OG', 'BV')

    def __init__(self, gateway, spc_area):
        self._gateway = gateway
        self._id = spc_area['id']
        self._name = spc_area['name']
        self.zones = None

        self.update(spc_area)

    def __str__(self):
        return '{id}: {name}. Mode: {mode}, last changed by {last}.'.format(
            name=self.name, id=self.id,
            mode=self.mode, last=self.last_changed_by)

    @property
    def id(self):
        return self._id

    @property
    def name(self):
        return self._name

    @property
    def mode(self):
        return self._mode

    @property
    def last_changed_by(self):
        return self._last_changed_by

    def update(self, spc_area):
        _LOGGER.debug("Update area %s", self.id)

        self._mode = _load_enum(AreaMode, spc_area['mode'])
        if self._mode == AreaMode.UNSET:
            self._last_changed_by = spc_area.get('last_unset_user_name', 'N/A')
        else:
            self._last_changed_by = spc_area.get('last_set_user_name', 'N/A')



from enum import Enum


class AreaMode(Enum):
    UNSET = '0'
    PART_SET_A = '1'
    PART_SET_B = '2'
    FULL_SET = '3'


class ZoneInput(Enum):
    CLOSED = '0'
    OPEN = '1'
    SHORT = '2'
    DISCONNECTED = '3'
    PIRMASKED = '4'
    DC_SUBSTITUTION = '5'
    SENSOR_MISSING = '6'
    OFFLINE = '7'


class ZoneType(Enum):
    ALARM = '0'
    ENTRY_EXIT = '1'
    EXIT_TERMINATOR = '2'
    FIRE = '3'
    FIRE_EXIT = '4'
    LINE = '5'
    PANIC = '6'
    HOLD_UP = '7'
    TAMPER = '8'
    TECHNICAL = '9'
    MEDICAL = '10'
    KEYARM = '11'
    UNUSED = '12'
    SHUNT = '13'
    X_SHUNT = '14'
    FAULT = '15'
    LOCK_SUPERVISION = '16'
    SEISMIC = '17'
    ALL_OKAY = '18'


class ZoneStatus(Enum):
    OK = '0'
    INHIBIT = '1'
    ISOLATE = '2'
    SOAK = '3'
    TAMPER = '4'
    ALARM = '5'
    OK_NOT_RECENT = '6'
    TROUBLE = '7'



"""Python wrapper for the Lundix SPC Web Gateway REST API."""
import logging
from urllib.parse import urljoin
from asyncio import ensure_future

from .area import Area
from .zone import Zone
from .const import AreaMode
from .utils import async_request
from .websocket import AIOWSClient

_LOGGER = logging.getLogger(__name__)


class SpcWebGateway:
    """Alarm system representation."""

    def __init__(self, loop, session, api_url, ws_url, async_callback):
        """Initialize the client."""
        self._loop = loop
        self._session = session
        self._api_url = api_url
        self._ws_url = ws_url
        self._areas = {}
        self._zones = {}
        self._websocket = None
        self._async_callback = async_callback

    @property
    def areas(self):
        """Retrieve all available areas."""
        return self._areas

    @property
    def zones(self):
        """Retrieve all available zones."""
        return self._zones

    def start(self):
        """Connect websocket to SPC Web Gateway."""
        self._websocket = AIOWSClient(loop=self._loop,
                                      session=self._session,
                                      url=self._ws_url,
                                      async_callback=self._async_ws_handler)
        self._websocket.start()

    async def async_load_parameters(self):
        """Fetch area and zone info from SPC to initialize."""
        zones = await self._async_get_data('zone')
        areas = await self._async_get_data('area')

        if not zones or not areas:
            return False

        for spc_area in areas:
            area = Area(self, spc_area)
            area_zones = [Zone(area, z) for z in zones
                          if z['area'] == spc_area['id']]
            area.zones = area_zones
            self._areas[area.id] = area
            self._zones.update({z.id: z for z in area_zones})

        return True

    async def change_mode(self, area, new_mode):
        """Set/unset/part set an area."""
        if not isinstance(new_mode, AreaMode):
            raise TypeError("new_mode must be an AreaMode")

        AREA_MODE_COMMAND_MAP = {
            AreaMode.UNSET: 'unset',
            AreaMode.PART_SET_A: 'set_a',
            AreaMode.PART_SET_B: 'set_b',
            AreaMode.FULL_SET: 'set'
        }
        if isinstance(area, Area):
            area_id = area.id
        else:
            area_id = area

        url = urljoin(self._api_url, "spc/area/{area_id}/{command}".format(
            area_id=area_id, command=AREA_MODE_COMMAND_MAP[new_mode]))

        return await async_request(self._session.put, url)

    async def _async_ws_handler(self, data):
        """Process incoming websocket message."""
        sia_message = data['data']['sia']
        spc_id = sia_message['sia_address']
        sia_code = sia_message['sia_code']

        _LOGGER.debug("SIA code is %s for ID %s", sia_code, spc_id)

        if sia_code in Area.SUPPORTED_SIA_CODES:
            entity = self._areas.get(spc_id, None)
            resource = 'area'
        elif sia_code in Zone.SUPPORTED_SIA_CODES:
            entity = self._zones.get(spc_id, None)
            resource = 'zone'
        else:
            _LOGGER.debug("Not interested in SIA code %s", sia_code)
            return
        if not entity:
            _LOGGER.error("Received message for unregistered ID %s", spc_id)
            return

        data = await self._async_get_data(resource, entity.id)
        entity.update(data)

        if self._async_callback:
            ensure_future(self._async_callback(entity))

    async def _async_get_data(self, resource, id=None):
        """Get the data from the resource."""
        if id:
            url = urljoin(self._api_url, "spc/{}/{}".format(resource, id))
        else:
            url = urljoin(self._api_url, "spc/{}".format(resource))
        data = await async_request(self._session.get, url)
        if not data:
            return False
        if id and isinstance(data['data'][resource], list):
            # for some reason the gateway returns an array with a single
            # element for areas but not for zones...
            return data['data'][resource][0]
        elif id:
            return data['data'][resource]

        return [item for item in data['data'][resource]]
